import os
import pandas as pd

from hiv_workflow.lib.analysis.download_analyzer_by_experiment import DownloadAnalyzerByExperiment
from hiv_workflow.lib.analysis.download_analyzer_by_experiment_receipt import DownloadAnalyzerByExperimentReceipt

from idmtools.analysis.analyze_manager import AnalyzeManager
from idmtools.core import ItemType
from idmtools.core.platform_factory import Platform


def download_experiments(analyzer, experiment_ids, platform):
    am = AnalyzeManager(ids=[(exp_id, ItemType.EXPERIMENT) for exp_id in experiment_ids],
                        analyzers=[analyzer],
                        verbose=False,
                        platform=platform)
    am.analyze()


def main(args):
    platform = Platform(args.platform)
    if args.receipt_file:
        receipt = pd.read_csv(args.receipt_file, index_col='index')
        experiment_ids = receipt['experiment_id'].unique()
        experiments = [platform.get_item(item_id=experiment_id, item_type=ItemType.EXPERIMENT)
                       for experiment_id in experiment_ids]
        analyzer = DownloadAnalyzerByExperimentReceipt(filenames=args.files, receipt_file=args.receipt_file)
    else:
        experiments = platform.get_children(item_id=args.suite_id, item_type=ItemType.SUITE)
        experiment_ids = [experiment.id for experiment in experiments]
        analyzer = DownloadAnalyzerByExperiment(filenames=args.files, output_path=args.output_dir)

    for experiment in experiments:
        platform.wait_till_done(item=experiment)
    download_experiments(analyzer=analyzer, experiment_ids=experiment_ids, platform=platform)
    print(f'Done downloading files to: {analyzer.output_path}')


DEFAULTS = {
    'platform': 'Calculon',
    'files': os.path.join('output', 'ReportHIVByAgeAndGender.csv'),
    'suite_id': None,
    'output_dir': None,
    'receipt_file': None
}


def parse_args():
    import argparse
    parser = argparse.ArgumentParser()

    parser.add_argument('-f', '--files', dest='files', type=str, default=DEFAULTS['files'],
                        help=f"Comma-separated list of simulation directory relative file paths to download "
                             f"(Default: {DEFAULTS['files']}")
    parser.add_argument('-r', '--receipt', dest='receipt_file', type=str, default=DEFAULTS['receipt_file'],
                        help='Commissioning receipt file path (Either set this OR use -id and -o).')
    parser.add_argument('-id', '--suite-id', dest='suite_id', type=str, default=DEFAULTS['suite_id'],
                        help='Id of suite to download simulations from (Either set this and -o OR use -r')
    parser.add_argument('-o', '--output-dir', dest='output_dir', type=str, default=DEFAULTS['output_dir'],
                        help='Directory to write output into (Must be set if using -id, do not set with -r)')
    parser.add_argument('-p', '--platform', dest='platform', type=str, default=DEFAULTS['platform'],
                        help=f"Platform to download from (Default: {DEFAULTS['platform']})")

    args = parser.parse_args()
    args.files = args.files.strip().split(',')

    # arg validation
    if args.receipt_file:
        if args.suite_id or args.output_dir:
            raise ValueError('-id and -o flags are not compatible with -r .')
        args.output_dir = os.path.dirname(os.path.abspath(args.receipt_file))
    else:
        if not args.suite_id or not args.output_dir:
            raise ValueError('Must run with -r OR -id and -o .')
    return args


if __name__ == '__main__':
    args = parse_args()
    main(args)
